// Copyright https://github.com/MothCocoon/FlowGraph/graphs/contributors

#pragma once

#include "DetailsDiff.h"
#include "DiffResults.h"
#include "IAssetTypeActions.h"
#include "Kismet/Private/DiffControl.h"

struct FDiffResultItem;
class UEdGraph;
struct FEdGraphEditAction;

class UFlowAsset;
class SFlowDiff;

/////////////////////////////////////////////////////////////////////////////
/// FFlowAssetDiffControl
class FLOWEDITOR_API FFlowAssetDiffControl : public TDetailsDiffControl<false>
{
public:
	FFlowAssetDiffControl(const UFlowAsset* InOldFlowAsset, const UFlowAsset* InNewFlowAsset, FOnDiffEntryFocused InSelectionCallback);

	virtual void GenerateTreeEntries(TArray<TSharedPtr<FBlueprintDifferenceTreeEntry>>& OutTreeEntries, TArray<TSharedPtr<FBlueprintDifferenceTreeEntry>>& OutRealDifferences) override;
};

/////////////////////////////////////////////////////////////////////////////
/// FFlowGraphToDiff: engine's FGraphToDiff customized to Flow Graph
struct FLOWEDITOR_API FFlowGraphToDiff : public TSharedFromThis<FFlowGraphToDiff>, IDiffControl
{
	FFlowGraphToDiff(class SFlowDiff* DiffWidget, UEdGraph* GraphOld, UEdGraph* GraphNew, const FRevisionInfo& RevisionOld, const FRevisionInfo& RevisionNew);
	virtual ~FFlowGraphToDiff() override;

	/** Add widgets to the differences tree */
	virtual void GenerateTreeEntries(TArray<TSharedPtr<FBlueprintDifferenceTreeEntry>>& OutTreeEntries, TArray<TSharedPtr<FBlueprintDifferenceTreeEntry>>& OutRealDifferences) override;

	UEdGraph* GetGraphOld() const { return GraphOld; };
	UEdGraph* GetGraphNew() const { return GraphNew; };

	/** Source for list view */
	TArray<TSharedPtr<FDiffResultItem>> DiffListSource;
	TSharedPtr<TArray<FDiffSingleResult>> FoundDiffs;

	/** Index of the first item in RealDifferences that was generated by this graph */
	int32 RealDifferencesStartIndex = INDEX_NONE;

private:
	FText GetToolTip() const;
	TSharedRef<SWidget> GenerateCategoryWidget() const;

	/** Called when the Newer Graph is modified*/
	void OnGraphChanged(const FEdGraphEditAction& Action) const;

	void BuildDiffSourceArray();

	class SFlowDiff* DiffWidget;
	UEdGraph* GraphOld;
	UEdGraph* GraphNew;

	/** Description of Old and new graph */
	FRevisionInfo RevisionOld;
	FRevisionInfo RevisionNew;

	FDelegateHandle OnGraphChangedDelegateHandle;
};
